"""Import Tevatron utility functions."""
from prompt2model.utils.tevatron_utils.encode import encode_text
from prompt2model.utils.tevatron_utils.retrieve import retrieve_objects

__all__ = ("encode_text", "retrieve_objects")
